// -*- java -*-
// ICSTARS: Defensive Programming
// Author: ks@kurtstephens.com
// Revision: $Id: Account5.cs,v 1.1 2002/07/10 05:27:07 stephens Exp $
//

// This program models a bank account.
// It should allow deposits and withdrawls.
// It has a number of problems.
//
// 1) Insert debugging traces to see what the program
// is doing wrong, try to determine what might be causing
// the problems.
//
// 2) Insert pre-condition assertions to catch
// any interface defects.
//

namespace ICSTARS {
    using System;
    
    // This is a bank account class.
    public class Account
    {
	// All accounts start with a zero balance.
	static double currentBalance = 0;

	// Get the balance of the account.
	public static double getCurrentBalance() 
	{
	    Debug.methodBegin("Account", "getCurrentBalance");
	    return (double) Debug.methodEnd("Account", "getCurrentBalance", currentBalance);
	}

	// Print the balance of the account to the console.
	public static void printCurrentBalance() 
	{
	    Debug.methodBegin("Account", "printCurrentBalance");

	    Console.WriteLine("Current Balance = {0}", getCurrentBalance());

	    Debug.methodEnd("Account", "printCurrentBalance");
	}

	// Deposit an amount in the account.
	// The amount must be greater than zero.
	public static void deposit(double amount)
	{
	    // Add amount to the current balance.

	    Debug.methodBegin("Account", "deposit", amount);

	    Debug.assert("amount > 0", amount > 0);

	    currentBalance = currentBalance + amount;

	    Debug.methodEnd("Account", "deposit", currentBalance);
	}

	// Withdraw an amount from the account.
	// The amount must be greater than zero.
	public static void withdraw(double amount)
	{
	    // Subtract amount from the current balance.
	    Debug.methodBegin("Account", "withdraw", amount);

	    Debug.assert("amount > 0", amount > 0);

	    currentBalance = currentBalance - amount;

	    Debug.methodEnd("Account", "withdraw", currentBalance);
	}
    }
    

public class AccountApp
{
    // Here is the main program.
    public static void Main(string[] args)
    {	
	Debug.methodPrintOn = false;
	Debug.methodBegin("AccountApp", "Main");

	// Here is a list of the account transactions.
	//      trans.,     amount
	Account.deposit (   100.50);
	Account.withdraw(    24.00);
	Account.withdraw(    25.00);
	Account.withdraw(   120.00);
	Account.deposit (    75.00);
	Account.deposit (    23.35);
	Account.deposit (    12.34);
        Account.deposit (   500.00);
	Account.withdraw(  1000.00);
	Account.deposit (   250.00);

	// Print the account's current balance.
	Account.printCurrentBalance();

	Debug.methodEnd("AccountApp", "Main");
    }
}


}
