// -*- java -*-
// ICSTARS: Defensive Programming
// Author: ks@kurtstephens.com
// Revision: $Id: AccountAvg.cs,v 1.1 2002/05/13 21:38:35 stephens Exp $
//

// This program models a bank account.
// It should allow deposits and withdrawls.
// It should also keep track of the balance of each day
// of a 31-day month.
// It should should allow a average daily balance to be 
// computed at any time.
// It has a number of problems.
// Insert debugging traces to see what the program
// is doing wrong, try to determine what might be causing
// the problems.
//

namespace ICSTARS {
    using System;
    
    public class AccountAvg
    {
	// Start on the first day of the month.
	static int currentDate = 1;

	// All accounts start with a zero balance.
	static double currentBalance = 0;

	// Keep track of the balance at each day of the month.
	// Assume the month has 31 days (1 - 31, inclusive).
	static double[] balanceAtDayOfTheMonth = new double[32];

	// Get the balance from the account.
	public static double getCurrentBalance() 
	{ 
	    return currentBalance;
	}

	// Print the balance of the account to the console.
	public static void printCurrentBalance() 
	{
	    Console.WriteLine("Current Balance = {0}", currentBalance);
	}

	public static void printDailyBalances()
	{
	    for ( int dayOfTheMonth = 1; dayOfTheMonth < 30; ++ dayOfTheMonth ) {
		Console.WriteLine("Day {0} Balance = {1}", dayOfTheMonth, balanceAtDayOfTheMonth[dayOfTheMonth]);
	    }
	}

	public static void printAverageDailyBalance()
	{
	    double dailyBalanceSum = 0;

	    for ( int dayOfTheMonth = 1; dayOfTheMonth < 31; ++ dayOfTheMonth ) {
		dailyBalanceSum = dailyBalanceSum + balanceAtDayOfTheMonth[dayOfTheMonth];
	    }

	    double averageDailyBalance = dailyBalanceSum / 31;
	    Console.WriteLine("Average Daily Balance = {0}", averageDailyBalance );
	}


	// Deposit an amount in the account.
	// The amount must be greater than zero.
	public static void deposit(int dayOfDeposit, double amount)
	{
	    int dayOfTheMonth;

	    for ( dayOfTheMonth = currentDate; dayOfTheMonth <= dayOfDeposit; ++ dayOfTheMonth ) {
		balanceAtDayOfTheMonth[dayOfTheMonth] = currentBalance;
	    }

	    currentDate = dayOfTheMonth;
	    currentBalance = currentBalance + amount;

	    for ( dayOfTheMonth = currentDate; dayOfTheMonth < 31; ++ dayOfTheMonth ) {
		balanceAtDayOfTheMonth[dayOfTheMonth] = currentBalance;
	    }
	}

	// Withdraw an amount from the account.
	// The amount must be greater than zero.
	public static void withdraw(int dayOfWithdrawl, double amount)
	{
	    int dayOfTheMonth;

	    for ( dayOfTheMonth = currentDate; dayOfTheMonth < dayOfWithdrawl; ++ dayOfTheMonth ) {
		balanceAtDayOfTheMonth[dayOfTheMonth] = currentBalance;
	    }

	    currentDate = dayOfTheMonth;
	    currentBalance = currentBalance - amount;

	    for ( dayOfTheMonth = currentDate; dayOfTheMonth < 31; ++ dayOfTheMonth ) {
		balanceAtDayOfTheMonth[dayOfTheMonth] = currentBalance;
	    }
	}
    }
    

public class AccountAvgApp
{
    // Here is the main program.
    public static void Main(string[] args)
    {	
	//      trans.,  day,  amount
	AccountAvg.deposit (1,    100.50);
	AccountAvg.withdraw(1,    -24.00);
	AccountAvg.withdraw(2,    -25.00);
	AccountAvg.withdraw(4,   -120.00);
	AccountAvg.deposit (8,     75.00);
	AccountAvg.deposit (8,     23.35);
	AccountAvg.deposit (12,   -12.34);
	AccountAvg.withdraw(21, -1000.00);
	AccountAvg.deposit (19,   250.00);

	AccountAvg.printDailyBalances();
	AccountAvg.printCurrentBalance();
	AccountAvg.printAverageDailyBalance();
    }
}


}
